/*
#
#    Copyright (C) 1999  Eric S. Tiedemann
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
*/

/*
  TODO: thread-safe version
*/

#include <stdio.h>
#include <math.h>

#ifdef PYTHON
#include <python1.5/Python.h>
#endif

static char buf[100];

#ifdef PYTHON
static
#endif
const char *
float2hms(double f)
{
  int h, m, s, c;
  int negp = f < 0;

  /* this should be round() */
  f = floor(fabs(f) * 100);

  h = (int) floor(f / 360000);
  f = f - h * 360000;
  m = (int) floor(f / 6000);
  f = f - m * 6000;
  s = (int) floor(f / 100);
  f = f - s * 100;
  c = (int) floor(f);
  if (h == 0)
    {
      if (negp)
	sprintf(buf, "   -%02d:%02d.%02d", m, s, c);
      else
	sprintf(buf, "    %02d:%02d.%02d", m, s, c);
    }
  else
    {
      if (negp)
	sprintf(buf, "-%02d:%02d:%02d.%02d", h, m, s, c);
      else
	sprintf(buf, " %02d:%02d:%02d.%02d", h, m, s, c);
    }

  return buf;
}

#ifdef PYTHON

static PyObject *
pyfloat2hms(PyObject *self, PyObject *args)
{
  double d;

  if (!PyArg_ParseTuple(args, "d;float2hms.float2hms", &d))
    return NULL;

  return Py_BuildValue("s", float2hms(d));
}

static PyMethodDef float2hmsMethods[] =
{
  {"float2hms", pyfloat2hms, METH_VARARGS},
  {0, 0, 0}
};

void
initfloat2hms()
{
  Py_InitModule("float2hms", float2hmsMethods);
}

#endif

/*#define TEST*/
#ifdef TEST

#include <stdlib.h>

int
main(int argc, char **argv)
{
  double f;
  
  if (argc != 2)
    return 1;

  f = atof(argv[1]);

  printf("%s\n", float2hms(f));

  return 0;
  }
#endif
